 /********************************************************************/
 /*                                                                  */
 /* Program name: amqsldpc                                           */
 /*                                                                  */
 /* Description: Sample C program that queries an LDAP directory     */
 /*              and then puts messages to a message queue           */
 /*              (example using MQPUT)                               */
 /*                                                                  */
 /*  Statement:     Licensed Materials - Property of IBM             */
 /*                                                                  */      
 /*                 (C) Copyright IBM Corp. 1998                     */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*   amqsldpc is a sample C program that looks up the MQ host name, */
 /*   channel name and queue name in an LDAP directory, and then     */
 /*   puts a message onto the named queue.                           */
 /*                                                                  */
 /*      -- messages are sent to the queue named in the LDAP         */
 /*         directory                                                */
 /*                                                                  */
 /*      -- gets lines from StdIn, and adds each to target           */
 /*         queue, taking each line of text as the content           */
 /*         of a datagram message; the sample stops when a null      */
 /*         line (or EOF) is read.                                   */
 /*         New-line characters are removed.                         */
 /*         If a line is longer than 99 characters it is broken up   */
 /*         into 99-character pieces. Each piece becomes the         */
 /*         content of a datagram message.                           */
 /*         If the length of a line is a multiple of 99 plus 1       */
 /*         e.g. 199, the last piece will only contain a new-line    */
 /*         character so will terminate the input.                   */
 /*                                                                  */
 /*      -- writes a message for each MQI reason other than          */
 /*         MQRC_NONE; stops if there is a MQI completion code       */
 /*         of MQCC_FAILED                                           */
 /*                                                                  */
 /*    Program logic:                                                */
 /*         open and bind to the LDAP directory                      */
 /*         use the application name to retrieve the MQ details      */
 /*         unbind from the directory                                */
 /*         MQCONNX to the queue manager                             */ 
 /*         MQOPEN target queue for OUTPUT                           */
 /*         while end of input file not reached,                     */
 /*         .  read next line of text                                */
 /*         .  MQPUT datagram message with text line as data         */
 /*         MQCLOSE target queue                                     */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*   amqsldpc has one required parameter                            */
 /*                 - the MQ application name                        */
 /*                                                                  */
 /*   The LDAP directory search is controlled by 3 environment       */
 /*   variables, one compulsory and two optional.                    */
 /*                                                                  */
 /*   The compulsory environment variable is:                        */
 /*      LDAP_BASEDN  = the base distinguished name for the search   */
 /*                                                                  */
 /*   The optional environment variables are:                        */
 /*      LDAP_HOST    = name of the host running the LDAP server,    */
 /*                     defaults to the local host if not specified  */
 /*      LDAP_VERSION = the LDAP version number, either 2 or 3,      */
 /*                     defaults to version 2 if not specified       */
 /*                                                                  */
 /********************************************************************/

 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>

 /* includes for ldap */
 #include <ldap.h>

 /* includes for MQI */
 #include <cmqc.h>
 #include <cmqxc.h>                 /* For MQCD definition           */

 /* function prototype */
 static void sendMessages(char *, char *, char *, char *);
 
 /* define the names of the LDAP attributes we need */
 #define MQ_HOST_ATTR               "host"
 #define MQ_CHANNEL_ATTR            "mqChannel"
 #define MQ_QUEUE_ATTR              "mqQueue"
 #define MQ_QUEUE_MANAGER_ATTR      "mqQueueManager"
 #define MQ_PORT_ATTR               "mqPort"
 /* prefix for the LDAP search filter */
 #define FILT_PREFIX                "cn="

 /* this is a MQ Client program */
 #define MQ_CLIENT

 /* store the attribute names in an array */
 static char* attrs[]    = {MQ_HOST_ATTR,
			    MQ_CHANNEL_ATTR,
			    MQ_QUEUE_ATTR, 
			    MQ_QUEUE_MANAGER_ATTR, 
			    MQ_PORT_ATTR, 
			    NULL};

 /********************************************************************/
 /*                                                                  */
 /*  main routine                                                    */
 /*                                                                  */
 /********************************************************************/
 void main(int argc, char **argv)
 {
   char  filterPattern[1024];      /* buffer for the filter pattern  */
   char	*bindDN   = NULL;          /* bind anonymously               */
   char	*password = NULL;          /* no password for anonymous bind */
   char	*baseDN;                   /* base distinguished name        */
   char	*ldapHost;                 /* LDAP server hostname           */
   char *versionNo;                /* LDAP protocol version          */
   char	*attribute;                /* LDAP attribute name            */
   char **values;                  /* LDAP attribute values          */
   char *mqHost         = NULL;    /* MQ server host name            */
   char *mqChannel      = NULL;    /* MQ Channel name                */
   char *mqQueue        = NULL;    /* MQ Queue name                  */
   char *mqQueueManager = NULL;    /* MQ Queue Manager name          */
   char *mqPort         = 0;       /* MQ port number                 */
   int	rc;                        /* return codes                   */ 
   int  ldapVersion;               /* LDAP protocol version          */
   LDAP  *ld;                      /* LDAP handle                    */
   LDAPMessage  *ldapResult;       /* LDAP search result             */
   LDAPMessage  *ldapEntry;        /* entry from the search result   */
   BerElement   *ber;              /* Basic Encodng Routine element  */
 
   /******************************************************************/
   /*                                                                */
   /*  Check the program has been called correctly and set up        */  
   /*  the search filter pattern from the command line argument      */
   /*                                                                */
   /******************************************************************/
   if (argc != 2)
   {
     printf("usage: %s  MQapplication\n", argv[0] );
     exit( 1 );
   }

   printf("Sample amqsldpc start\n");
 
   strcpy(filterPattern, FILT_PREFIX);
   strcat(filterPattern, argv[1]);
 
   /******************************************************************/
   /*                                                                */
   /*  Check Environment variables                                   */  
   /*                                                                */
   /******************************************************************/
   baseDN = getenv("LDAP_BASEDN");
   if (baseDN == NULL)
   {
     printf("\"LDAP_BASEDN\" environment variable is not set.\n");
     exit(1);
   }
 
   ldapHost = getenv("LDAP_HOST");
   if (ldapHost == NULL)
   {
     ldapHost = "localhost";            /* default to the local host */
   }
 
   versionNo = getenv("LDAP_VERSION");
   if (versionNo != NULL)
   {
     ldapVersion = atol(versionNo);
     if (ldapVersion != LDAP_VERSION2 && ldapVersion != LDAP_VERSION3)
     {
       printf(
           "\"LDAP_VERSION\" specifies an invalid version number.\n");
       exit(1);
     }
   }
   else
     ldapVersion = LDAP_VERSION2;            /* default to version 2 */
 
   /******************************************************************/
   /*                                                                */
   /*  Initialise the connection to the LDAP server                  */  
   /*                                                                */
   /******************************************************************/
   if (ldapVersion == LDAP_VERSION3)
   {
     if ((ld = ldap_init(ldapHost, LDAP_PORT)) == NULL)
     {
       perror(ldapHost);
       exit(1);
     }
   } 
   else
   {
     if ((ld = ldap_open(ldapHost, LDAP_PORT)) == NULL )
     {
       perror(ldapHost);
       exit(1);
     }
   }
 
   /******************************************************************/
   /*                                                                */
   /*  Set LDAP options:                                             */  
   /*      - the LDAP version number                                 */  
   /*      - LDAP client does not follow referrals                   */  
   /*                                                                */
   /******************************************************************/
   ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, (void *)&ldapVersion);
   ldap_set_option(ld, LDAP_OPT_REFERRALS, (void *)LDAP_OPT_OFF);
 
   /******************************************************************/
   /*                                                                */
   /*  Bind to the LDAP server                                       */  
   /*                                                                */
   /******************************************************************/
   if (ldapVersion == LDAP_VERSION3)
   {
     if (ldap_simple_bind_s(ld, bindDN, password) != LDAP_SUCCESS)
     {
       ldap_perror(ld, "ldap_simple_bind");/* print an error message */
       exit(1);
     }
   } 
   else
   {
     if (ldap_bind_s(ld, bindDN, password, LDAP_AUTH_SIMPLE) != 
	 LDAP_SUCCESS)
     {
       ldap_perror(ld, "ldap_bind");       /* print an error message */
       exit(1);
     }
   }
 
   /******************************************************************/
   /*                                                                */
   /*  Search the LDAP directory. For this example, a simple         */
   /*  synchronous search is used.                                   */  
   /*                                                                */
   /******************************************************************/
   rc = ldap_search_s(ld,                 /* LDAP Handle             */
 		     baseDN,              /* base distinguished name */
 		     LDAP_SCOPE_ONELEVEL, /* one-level search        */
 		     filterPattern,       /* filter search pattern   */
 		     attrs,               /* attributes required     */
 		     FALSE,               /* NOT attributes only     */
 		     &ldapResult);        /* search result           */
 
   if (rc != LDAP_SUCCESS)
   {
     ldap_perror(ld, "ldap_search");      /* print an error message  */
     exit(1);
   }
 
   /******************************************************************/
   /*                                                                */
   /*  Extract the results. For this example we are only interested  */
   /*  in the first entry returned.                                  */
   /*                                                                */
   /******************************************************************/
   ldapEntry = ldap_first_entry(ld, ldapResult);
 
   if (ldapEntry == NULL)
   {                                     /* print an error message  */
     printf("Application \"%s\" not found\n", argv[1]);
     exit(1);
   }
 
   /******************************************************************/
   /*                                                                */
   /*  Extract the attribute values we need from the entry.          */
   /*                                                                */
   /******************************************************************/
   for (attribute = ldap_first_attribute(ld, ldapEntry, &ber);
        attribute != NULL; 
        attribute = ldap_next_attribute(ld, ldapEntry, ber ))
   {
     values = ldap_get_values(ld, ldapEntry, attribute);
     if (values != NULL && values[0] != NULL)
     {
       if (stricmp(attribute, MQ_HOST_ATTR) == 0)
	 mqHost = strdup(values[0]);
       else if (stricmp(attribute, MQ_CHANNEL_ATTR) == 0)
	 mqChannel = strdup(values[0]);
       else if (stricmp(attribute, MQ_QUEUE_ATTR) == 0)
	 mqQueue = strdup(values[0]);
       else if (stricmp(attribute, MQ_QUEUE_MANAGER_ATTR) == 0)
	 mqQueueManager = strdup(values[0]);
       else if (stricmp(attribute, MQ_PORT_ATTR) == 0)
	 mqPort = strdup(values[0]);
 
       /**************************************************************/
       /*                                                            */
       /*  Free the values.                                          */
       /*                                                            */
       /**************************************************************/
       ldap_value_free(values);
     }
 
     /****************************************************************/
     /*                                                              */
     /*  Free the attribute.                                         */
     /*                                                              */
     /****************************************************************/
     ldap_memfree(attribute);
   }
 
   /******************************************************************/
   /*                                                                */
   /*  Free the message.                                             */
   /*                                                                */
   /******************************************************************/
   ldap_msgfree(ldapResult);
   
   /******************************************************************/
   /*                                                                */
   /*  unbind from the directory.                                    */
   /*                                                                */
   /******************************************************************/
   ldap_unbind(ld);

 #ifdef VERBOSE 
   /* print the results */
   printf("MQ Host is %s\n", 
	  mqHost == NULL ? "<Unkown>" : mqHost);
   printf("MQ Channel is %s\n", 
	  mqChannel == NULL ? "<Unkown>" : mqChannel);
   printf("MQ Queue is %s\n", 
	  mqQueue == NULL ? "<Unkown>" : mqQueue);
   printf("MQ Queue Manager is %s\n", 
	  mqQueueManager == NULL ? "<default>" : mqQueueManager);
   printf("MQ Port is %s\n", mqPort == NULL ? "<default>" : mqPort);
 #endif

   /******************************************************************/
   /*                                                                */
   /*  check that we've got all the values we need.                  */
   /*                                                                */
   /******************************************************************/
 #ifdef MQ_CLIENT
   if (mqHost == NULL)
     printf("Unable to find the MQ server (host) name\n");
   else if (mqChannel == NULL)
     printf("Unable to find the MQ Channel name\n");
   else 
 #endif
   if (mqQueue == NULL)
     printf("Unable to find the MQ Queue name\n");
   else
   {
     /****************************************************************/
     /*                                                              */
     /* set the Host Connection variable and send the MQ messsages   */
     /*                                                              */
     /****************************************************************/
     char HostConn[1024];

     if (mqPort == 0)
       strcpy(HostConn, mqHost);
     else
       sprintf(HostConn, "%s(%s)", mqHost, mqPort);

     sendMessages(HostConn, mqChannel, mqQueue, mqQueueManager);
   }

   /******************************************************************/
   /*                                                                */
   /* end of amqsldpc                                                */
   /*                                                                */
   /******************************************************************/
   printf("Sample amqsldpc end\n");
 
 }


 /********************************************************************/
 /*                                                                  */
 /*  Connect to the MQ server and put the messages.                  */
 /*                                                                  */
 /********************************************************************/
 void sendMessages(char *HostConn,     /* Host Connection name       */
		   char *ChannelName,  /* MQ Channel Name            */
		   char *QName,        /* MQ Queue Name              */
		   char *QManagerName) /* MQ Queue Manager Name      */
 {
   /*  Declare file and character for sample input                   */
   FILE *fp;

   /*   Declare MQI structures needed                                */
   MQCNO    Connect_options = {MQCNO_DEFAULT}; /* MQCONNX options    */
   MQCD     ClientConn = {MQCD_CLIENT_CONN_DEFAULT};
                                    /* Client connection channel     */
                                    /* definition                    */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */
      /** note, sample uses defaults where it can **/

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   MQLONG   buflen;                 /* buffer length                 */
   char     buffer[100];            /* message buffer                */

   /******************************************************************/
   /*                                                                */
   /*   Save parameter as the name of the target queue               */
   /*                                                                */
   /******************************************************************/
   strncpy(od.ObjectName, QName, (size_t)MQ_Q_NAME_LENGTH);

   /******************************************************************/
   /*                                                                */
   /*   Initialise the client channel definition                     */
   /*                                                                */
   /******************************************************************/
   if (HostConn != NULL)
   {
     strncpy(ClientConn.ConnectionName, HostConn, MQ_CONN_NAME_LENGTH);

     if (ChannelName == NULL)
     {
       ChannelName = "SYSTEM.DEF.SVRCONN";
     }
     strncpy(ClientConn.ChannelName,
	     ChannelName,
	     MQ_CHANNEL_NAME_LENGTH);

     /* Point the MQCNO to the client connection definition */
     Connect_options.ClientConnPtr = &ClientConn;

     /* Client connection fields are in the version 2 part of the
        MQCNO so we must set the version number to 2 or they will
        be ignored */
     Connect_options.Version = MQCNO_VERSION_2;
   }

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   if (QManagerName == NULL)
     QManagerName = "";

   MQCONNX(QManagerName,           /* queue manager                  */
	   &Connect_options,       /* options for connection         */
	   &Hcon,                  /* connection handle              */
	   &CompCode,              /* completion code                */
	   &CReason);              /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     exit( (int)CReason );
   }

   /******************************************************************/
   /*                                                                */
   /*   Open the target message queue for output                     */
   /*                                                                */
   /******************************************************************/
   O_options = MQOO_OUTPUT           /* open queue for output        */
           + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping      */
   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* MQOPEN completion code       */
          &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN ended with reason code %ld\n", Reason);
   }

   if (OpenCode == MQCC_FAILED)
   {
     printf("unable to open queue for output\n");
   }

   /******************************************************************/
   /*                                                                */
   /*   Read lines from the file and put them to the message queue   */
   /*   Loop until null line or end of file, or there is a failure   */
   /*                                                                */
   /******************************************************************/
   CompCode = OpenCode;        /* use MQOPEN result for initial test */
   fp = stdin;

   memcpy(md.Format,        
  /* character string format         */
          MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);

   printf("Enter Messages:\n");
   while (CompCode != MQCC_FAILED)
   {
     if (fgets(buffer, sizeof(buffer), fp) != NULL)
     {
       buflen = strlen(buffer);       /* length without null         */
       if (buffer[buflen-1] == '\n')  /* last char is a new-line     */
       {
         buffer[buflen-1]  = '\0';    /* replace new-line with null  */
         --buflen;                    /* reduce buffer length        */
       }
     }
     else buflen = 0;        /* treat EOF same as null line          */

     /****************************************************************/
     /*                                                              */
     /*   Put each buffer to the message queue                       */
     /*                                                              */
     /****************************************************************/
     if (buflen > 0)
     {

       memcpy(md.MsgId,           /* reset MsgId to get a new one    */
              MQMI_NONE, sizeof(md.MsgId) );

       memcpy(md.CorrelId,        /* reset CorrelId to get a new one */
              MQCI_NONE, sizeof(md.CorrelId) );

       MQPUT(Hcon,                /* connection handle               */
             Hobj,                /* object handle                   */
             &md,                 /* message descriptor              */
             &pmo,                /* default options (datagram)      */
             buflen,              /* buffer length                   */
             buffer,              /* message buffer                  */
             &CompCode,           /* completion code                 */
             &Reason);            /* reason code                     */

       /* report reason, if any */
       if (Reason != MQRC_NONE)
       {
         printf("MQPUT ended with reason code %ld\n", Reason);
       }
     }
     else   /* satisfy end condition when empty line is read */
       CompCode = MQCC_FAILED;
   }

   /******************************************************************/
   /*                                                                */
   /*   Close the target queue (if it was opened)                    */
   /*                                                                */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
             &Hobj,                   /* object handle               */
             C_options,
             &CompCode,               /* completion code             */
             &Reason);                /* reason code                 */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("MQCLOSE ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %ld\n", Reason);
     }
   }
 }

